import { Component, inject, OnDestroy } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { Group } from '../../shared/models/group';
import { Policy } from '../../shared/models/policy';
import { MatExpansionModule } from '@angular/material/expansion';
import { PoliciesModule } from '../../policies/policies.module';
import { MatInputModule } from '@angular/material/input';
import { MatFormFieldModule } from '@angular/material/form-field';
import { ReactiveFormsModule } from '@angular/forms';
import { MatButtonModule } from '@angular/material/button';
import { MatIconModule } from '@angular/material/icon';
import { PolicyTableComponent } from "../../policies/policy-table/policy-table.component";
import { PolicyAddButtonComponent } from '../../policies/policy-add-button/policy-add-button.component';
import { HermesClientService } from '../../hermes-client.service';
import { GroupChatter } from '../../shared/models/group-chatter';
import { TwitchUsersModule } from "../../twitch-users/twitch-users.module";
import { SpecialGroups } from '../../shared/utils/groups';
import { PermissionListComponent } from "../../permissions/permission-list/permission-list.component";
import { Permission } from '../../shared/models/permission';
import { Subscription } from 'rxjs';
import { PermissionService } from '../../shared/services/permission.service';
import GroupService from '../../shared/services/group.service';
import PolicyService from '../../shared/services/policy.service';

@Component({
  imports: [
    MatButtonModule,
    MatExpansionModule,
    MatFormFieldModule,
    MatIconModule,
    MatInputModule,
    PoliciesModule,
    PolicyAddButtonComponent,
    ReactiveFormsModule,
    PolicyTableComponent,
    PolicyTableComponent,
    TwitchUsersModule,
    PermissionListComponent
  ],
  templateUrl: './group-page.component.html',
  styleUrl: './group-page.component.scss'
})
export class GroupPageComponent implements OnDestroy {
  private readonly _router = inject(Router);
  private readonly _route = inject(ActivatedRoute);
  private readonly _groupService = inject(GroupService);
  private readonly _permissionService = inject(PermissionService);
  private readonly _policyService = inject(PolicyService);
  private readonly _client = inject(HermesClientService);
  private _group: Group | undefined;
  private _chatters: GroupChatter[];
  private _policies: Policy[];
  private _permissions: Permission[];

  isSpecialGroup: boolean;
  _groups: Group[];

  private readonly subscriptions: (Subscription | undefined)[] = [];


  constructor() {
    this.isSpecialGroup = false
    this._groups = [];
    this._chatters = [];
    this._permissions = [];
    this._policies = [];

    this._route.params.subscribe((params: any) => {
      // Fetch the group id from the query params.
      const group_id = params['id'];

      this._route.data.subscribe(async (data: any) => {
        this._groups = data['groups'];
        const group = this.groups.find((g: Group) => g.id == group_id);

        if (!group) {
          await this._router.navigate(['groups']);
          return;
        }

        this._group = group;
        this.isSpecialGroup = SpecialGroups.includes(this.group!.name);
        this._chatters = data['chatters'];
        this._permissions = data['permissions'];
        this._policies = data['policies'];
      });
    });

    this.subscriptions.push(this._permissionService.delete$?.subscribe(d => {
      if (d.error) {
        return;
      }

      this._permissionService.fetch().subscribe(permissions => this._permissions = permissions);
    }));

    this.subscriptions.push(this._groupService.deleteGroup$?.subscribe(d => {
      if (d.error) {
        return;
      }

      this._groupService.fetch().subscribe(data => this._groups = data.groups);
    }));

    this.subscriptions.push(this._groupService.deleteChatter$?.subscribe(d => {
      if (d.error) {
        return;
      }

      this._groupService.fetch().subscribe(data => this._chatters = data.chatters);
    }));

    this.subscriptions.push(this._policyService.delete$?.subscribe(d => {
      if (d.error) {
        return;
      }

      this._policyService.fetch().subscribe(policies => this._policies = policies);
    }));
  }

  ngOnDestroy(): void {
    if (this.subscriptions) {
      for (let subscription of this.subscriptions) {
        if (subscription)
          subscription.unsubscribe();
      }
    }
  }

  get group() {
    return this._group;
  }

  get groups() {
    return this._groups;
  }

  get chatters() {
    if (!this._group) {
      return [];
    }
    return this._chatters.filter((c: GroupChatter) => c.group_id == this._group!.id);
  }

  get permissions() {
    if (!this._group) {
      return [];
    }
    return this._permissions.filter((p: Permission) => p.group_id == this._group!.id);
  }

  get policies() {
    if (!this._group) {
      return [];
    }
    return this._policies.filter((p: Policy) => p.group_id == this._group!.id);
  }

  delete() {
    if (!this.group)
      return;

    this._client.first(d => d.d.request.type == 'delete_group' && d.d.request.data.id == this.group!.id)
      .subscribe(async () => await this._router.navigate(['groups']));
    this._client.deleteGroup(this.group.id);
  }
}