import { AfterViewInit, Component, inject, Input, OnChanges, OnDestroy, OnInit, SimpleChanges, ViewChild } from '@angular/core';
import { MatTable, MatTableModule } from '@angular/material/table';
import { MatIconModule } from '@angular/material/icon';
import EventService from '../../shared/services/EventService';
import { Policy } from '../../shared/models/policy';
import { FormsModule } from '@angular/forms';
import { HermesClientService } from '../../hermes-client.service';
import { MatButtonModule } from '@angular/material/button';
import { ActivatedRoute } from '@angular/router';
import { MatDialog } from '@angular/material/dialog';
import { PolicyItemEditComponent } from '../policy-item-edit/policy-item-edit.component';
import { Group } from '../../shared/models/group';

@Component({
  selector: 'policy-table',
  imports: [FormsModule, MatButtonModule, MatTableModule, MatIconModule],
  templateUrl: './policy-table.component.html',
  styleUrl: './policy-table.component.scss'
})
export class PolicyTableComponent implements OnInit, OnDestroy, AfterViewInit {
  private readonly route = inject(ActivatedRoute);
  private readonly hermes = inject(HermesClientService);
  private readonly events = inject(EventService);
  private readonly dialog = inject(MatDialog);

  @Input() policies: Policy[] = [];
  @ViewChild(MatTable) table: MatTable<Policy>;

  readonly displayedColumns = ['path', 'group', 'usage', 'span', 'actions'];
  private readonly _subscriptions: any[] = [];

  groups: Group[] = [];


  constructor() {
    this.table = {} as MatTable<Policy>;
  }

  ngOnInit(): void {
    this.route.data.subscribe(r => {
      this.groups = [...r['groups']];
    });

    this._subscriptions.push(this.events.listen('addPolicy', (payload) => {
      if (!payload || this.policies.map(p => p.path).includes(payload))
        return;

      this.policies.push(payload);
      this.table.renderRows();
    }));

    this._subscriptions.push(this.hermes.subscribeToRequests('create_policy', response => {
      const policy = this.policies.find(p => p.path == response.data.path);
      if (policy == null) {
        this.policies.push(response.data);
      }
      this.table.renderRows();
    }));

    this._subscriptions.push(this.hermes.subscribeToRequests('update_policy', response => {
      const policy = this.policies.find(p => p.id == response.data.id);
      if (policy != null) {
        policy.id = response.data.id;
        policy.group_id = response.data.group_id;
      }
      this.table.renderRows();
    }));

    this._subscriptions.push(this.hermes.subscribeToRequests('delete_policy', response => {
      this.policies = this.policies.filter(p => p.id != response.request.data.id);
      this.table.renderRows();
    }));
  }

  ngAfterViewInit(): void {
    this.table.renderRows();
  }

  ngOnDestroy(): void {
    this._subscriptions.filter(s => !!s).forEach(s => s.unsubscribe());
  }

  delete(policy: Policy) {
    this.hermes.deletePolicy(policy.id);
  }

  edit(policy: Policy) {
    const dialogRef = this.dialog.open(PolicyItemEditComponent, {
      data: {
        policies: this.policies,
        groups: this.groups,
        policy_id: policy.id,
        group_id: policy.group_id,
        groupDisabled: true,
        isNew: false,
      }
    });

    dialogRef.afterClosed().subscribe((result: Policy) => {
      if (!result)
        return;

      policy.group_id = result.group_id;
      policy.path = result.path;
      policy.usage = result.usage;
      policy.span = result.span;
    });
  }

  getGroupById(group_id: string) {
    return this.groups.find((g: Group) => g.id == group_id);
  }
}